/*
/*  -ed  display edge maps (default false)
/*  -fd  display feature maps (default false)
/*  -cd  display contrast map (default false)
/*  -es  save edge maps (default false)
/*  -fs  save feature maps (default false)
/*  -s center_radius surround_radius set radii for contrast cell (default 7&12)
 */


#include <stdio.h>
#include <math.h>
#include <sys/ddi.h>
#include "pgm.h"
#include "edges.h"
#include "orient.h"
#include "utils.h"


main(int argc, char* argv[])
{
  unsigned char *image,
                *edge_map[ORIENTATIONS],
		*feature_map[ORIENTATIONS],
		*contrast_map;
  double *dbl_map[ORIENTATIONS];
  int width, height, max;
  int i, j, k;
  int centre=7, surround=12;
  int size;
  int e_display=0, f_display=0, c_display=0;
  int e_save=0, f_save=0;
  char file_name[40];
  char c_file[40];
  char syscall[60];
  char s_num[3];
  int num;
  int s_ind;
  int success;

  
  /* parse command line */

  if(argc<2)
  {
    fprintf(stderr, "USAGE: contrast [-em][-fm][-cm][-es][-fs]\n");
    fprintf(stderr, "       [-s center_radius surround_radius]");
    fprintf(stderr, " filename\n");
    exit(-1);
  }
  
  else
    parse(argc, argv, &e_display, &f_display, &c_display,
	  &e_save, &f_save, &centre, &surround, file_name);


  /* Read image file */
  max=read_pgm(file_name, &image, &width, &height);
  size=width*height;
    
 
  /* find edges */
  Edges(width, height, size, edge_map, image);


  /* write and display edge maps */
  if (e_save || e_display)
    save_disp(e_display, e_save, file_name, edge_map, width, height,
		max, "edge");


  /* allocate memory for feature and contrast maps */
  for (i=0; i<ORIENTATIONS; i++)
  {
    dbl_map[i]=(double *) calloc(4*size, sizeof(double));
    feature_map[i]=(unsigned char *) calloc(4*size, 1);
  }
  contrast_map=(unsigned char *) calloc(4*size,1);
  

  /* calculate contrast values */
  success = Orientation(width, height, size, centre, surround, 
		  edge_map, dbl_map, contrast_map);


  /* write and display feature maps */
  if (f_save || f_display)
  {
    for(i=0; i<ORIENTATIONS; i++)
      dbl_to_char(feature_map[i], dbl_map[i], size);
    save_disp(f_display, f_save, file_name, feature_map, width, height, 
		max, "feature");
  }

  
  /* save contrast map */
  strcpy(c_file, file_name);
  s_ind=strlen(c_file)-3;
  c_file[s_ind]=0;
  num_to_str(centre, s_num);
  strcat(c_file, s_num);
  strcat(c_file, ".");
  num_to_str(surround, s_num);
  strcat(c_file, s_num);
  strcat(c_file, ".orient.pgm");
  write_pgm(c_file, contrast_map, width, height, max);

  /* display contrast map */
  if (c_display)
  {
    strcpy(syscall, "xv ");
    strcat(syscall, c_file);
    strcat(syscall, " &");
    system(syscall);
  }

}


int Orientation(int width, int height, int size, 
	   int radius_centre, int radius,
	   unsigned char **edge_map, double **dbl_map,
	   unsigned char *c_map)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  static char file_name[]="c4.s06.o0.pgm",
	      contrast_name[]="c_map.c6.s06.pgm";
  float weight[ORIENTATIONS];
  double cell[ORIENTATIONS], max; 
  double **gaus_c, **gaus_s;

  FILE *f_ptr;

  gaus_c=(double **) calloc(2*radius_centre+1, sizeof(double*));
  for (i=0; i<2*radius_centre+1; i++)
    gaus_c[i]=(double *) calloc(2*radius_centre+1, sizeof(double));

  gaus_s=(double **) calloc(2*radius+1, sizeof(double*));
  for (i=0; i<2*radius+1; i++)
    gaus_s[i]=(double *) calloc(2*radius+1, sizeof(double));

  Calc_Weights(weight);

  Calc_Gaus(gaus_c, radius_centre);
  Calc_Gaus(gaus_s, radius);

  f_ptr=fopen("temp2", "w");

  for (pixel_y=radius; pixel_y<height-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<ORIENTATIONS; i++)
      {
	cell[i]=Convolve(pixel_x, pixel_y, width,
			 radius_centre, gaus_c, edge_map[i]);
	cell[i]=cell[i]-Convolve(pixel_x, pixel_y, width, radius, 
			  	 gaus_s, edge_map[i]);
      }


      for(i=0; i<ORIENTATIONS; i++)
	for(j=0; j<ORIENTATIONS; j++)
	{
	  dbl_map[i][pixel_y*width+pixel_x]+=
				weight[(j-i+ORIENTATIONS)%ORIENTATIONS]*cell[j];
	}

      fprintf(f_ptr, "%g ", dbl_map[0][pixel_y*width+pixel_x]);

    }/* pixels */

  fclose(f_ptr);

  max=Mult_Norm_dbl(dbl_map, size);

  if (max > 0.0)
    Combine_Maps(dbl_map, size, c_map);

  return max;
}


