#include <stdio.h>
#include <math.h>
#include "tls.h"
#include "pgm.h"

main(int argc, char* argv[])
{
  int width, height, max;
  int i, j;
  int size;
  unsigned char *feature_map[2];
  unsigned char *image;
  char base[20], fname[35];

  max=read_pgm(argv[1], &image, &width, &height);
  size=width*height;

  TLs(width, height, size, feature_map, image);

  strncpy(base, argv[1], strlen(argv[1])-3);
  base[strlen(argv[1])-3]=0;
  sprintf(fname, "%sT.pgm", base);
  write_pgm(fname, feature_map[0], width, height, max);
  sprintf(fname, "%sL.pgm", base);
  write_pgm(fname, feature_map[1], width, height, max);
}

void TLs(int width, int height, int size, unsigned char **edge_map1,
		unsigned char *image)
{
  int i, j, edge;
  int radius=2;
  int *temp_map1[ORIENTATIONS];
  char f_name[20];


  for (i=0; i<ORIENTATIONS; i++)
  {
    temp_map1[i]=(int *) calloc(4*size, sizeof(int));
    edge_map1[i]=(unsigned char *) calloc(4*size, 1);
  }

  printf("size=%d\n", size);

  for (edge=0; edge<2; edge++)
  {
    for (i=radius; i<height-radius; i++)
    {
      for (j=radius; j<width-radius; j++)
      {
        temp_map1[edge][i*width+j]=
				(int)abs(Sobel(i, j, edge, width, image));
      }
    }
  }

  MultipleNormalize(size, edge_map1, temp_map1);

}


float Sobel(int i, int j, int hierarchy, int width, unsigned char *image)
{
  float temp;

  static int T[49] =   {-10, -10, -10, -10, -10, -10, -10,
			-10, -10, -10, -10, -10, -10, -10,
			  6,   6,   1,   1,   1,   1,   1, 
			  6,   6,   1,   1,   1,   1,   1, 
			 -1,  -1,  11,  11,  -1,  -1,  -1,
			 -1,  -1,  11,  11,  -1,  -1,  -1,
			 -1,  -1,  11,  11,  -1,  -1,  -1};

  static int L[49] =   {-1,   -1,  11,  11,  -1,  -1,  -1,
			-1,   -1,  11,  11,  -1,  -1,  -1, 
			-1,   -1,  11,  11,  -1,  -1,  -1, 
			-10, -10,   1,   1,   1,   1,   1,
			-10, -10,   1,   1,   1,   1,   1,
			-1,   -1, -10, -10, -10, -10, -10,
			-1,   -1, -10, -10, -10, -10, -10};

  switch(hierarchy)
  {
    case 0: return Grad(i, j, width, T, image);
    case 1: return Grad(i, j, width, L, image);
  }     
}


float Grad(int i, int j, int width, int *val, unsigned char *image)
{
  float total=0;
  int x, y;

  i=i-3;
  j=j-3; 

  for(y=0; y<7; y++)
    for(x=0; x<7; x++)
      total+=(val[y*7+x]*image[(i+y)*width + (j+x)]);

  return total;

/*
  return(val[0]  * image[(i-2) * width + (j-2)] +
         val[1]  * image[(i-2) * width + (j-1)] +
         val[2]  * image[(i-2) * width +  (j) ] +
         val[3]  * image[(i-2) * width + (j+1)] +
         val[4]  * image[(i-2) * width + (j+2)] +
         val[5]  * image[(i-1) * width + (j-2)] +
         val[6]  * image[(i-1) * width + (j-1)] +
         val[7]  * image[(i-1) * width +  (j) ] +
         val[8]  * image[(i-1) * width + (j+1)] +
         val[9]  * image[(i-1) * width + (j+2)] +
         val[10] * image[ (i)  * width + (j-2)] +
         val[11] * image[ (i)  * width + (j-1)] +
         val[12] * image[ (i)  * width + (j)] +	
         val[13] * image[ (i)  * width + (j+1)] +
         val[14] * image[ (i)  * width + (j+2)] +
         val[15] * image[(i+1) * width + (j-2)] +
         val[16] * image[(i+1) * width + (j-1)] +
         val[17] * image[(i+1) * width +  (j) ] +
         val[18] * image[(i+1) * width + (j+1)] +
         val[19] * image[(i+1) * width + (j+2)] +
         val[20] * image[(i+2) * width + (j-2)] +
         val[21] * image[(i+2) * width + (j-1)] +
         val[22] * image[(i+2) * width +  (j) ] +
         val[23] * image[(i+2) * width + (j+1)] +
         val[24] * image[(i+2) * width + (j+2)] );
*/
}




void MultipleNormalize(int size, unsigned char **e_map, int **t_map)
{
  int i;
  float currMax, maxVal = 0.0;
  
  for(i=0; i< ORIENTATIONS; i++)
  {
    currMax = Maximum(i, size, t_map);
    if (currMax > maxVal)
       maxVal=currMax;
  }

  for(i=0; i< ORIENTATIONS; i++)

    Normalize(i, maxVal, size, e_map, t_map);

  return;
}  



void Normalize (int edge, int maxValue, int size, unsigned char **e_map,
		int **temp_map)
{
  int i;
  float multFac;

  multFac=255.0/maxValue;

  for(i=0; i<size; i++)
    e_map[edge][i]=(unsigned char)(temp_map[edge][i]*multFac);
}



float Maximum(int edge, int size, int **temp_map)
{
  int i;
  float max=0;

  for (i=0; i<size; i++)
  {
    if (temp_map[edge][i]>max)
      max=temp_map[edge][i];
  }

  return max;
}
